@@ -1,9 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -23,7 +23,7 @@
 
 
 static apr_hash_t *dav_liveprop_uris = NULL;
-static int dav_liveprop_count = 0;
+static long dav_liveprop_count = 0;
 
 
 static apr_status_t dav_cleanup_liveprops(void *ctx)
@@ -35,14 +35,14 @@ static apr_status_t dav_cleanup_liveprops(void *ctx)
 
 static void dav_register_liveprop_namespace(apr_pool_t *p, const char *uri)
 {
-    int value;
+    long value;
 
     if (dav_liveprop_uris == NULL) {
         dav_liveprop_uris = apr_hash_make(p);
         apr_pool_cleanup_register(p, NULL, dav_cleanup_liveprops, apr_pool_cleanup_null);
     }
 
-    value = (int)apr_hash_get(dav_liveprop_uris, uri, APR_HASH_KEY_STRING);
+    value = (long)apr_hash_get(dav_liveprop_uris, uri, APR_HASH_KEY_STRING);
     if (value != 0) {
         /* already registered */
         return;
@@ -53,12 +53,12 @@ static void dav_register_liveprop_namespace(apr_pool_t *p, const char *uri)
                  (void *)++dav_liveprop_count);
 }
 
-DAV_DECLARE(int) dav_get_liveprop_ns_index(const char *uri)
+DAV_DECLARE(long) dav_get_liveprop_ns_index(const char *uri)
 {
-    return (int)apr_hash_get(dav_liveprop_uris, uri, APR_HASH_KEY_STRING);
+    return (long)apr_hash_get(dav_liveprop_uris, uri, APR_HASH_KEY_STRING);
 }
 
-DAV_DECLARE(int) dav_get_liveprop_ns_count(void)
+DAV_DECLARE(long) dav_get_liveprop_ns_count(void)
 {
     return dav_liveprop_count;
 }
@@ -75,7 +75,7 @@ DAV_DECLARE(void) dav_add_all_liveprop_xmlns(apr_pool_t *p,
 
         apr_hash_this(idx, &key, NULL, &val);
 
-        s = apr_psprintf(p, " xmlns:lp%d=\"%s\"", (int)val, (const char *)key);
+        s = apr_psprintf(p, " xmlns:lp%ld=\"%s\"", (long)val, (const char *)key);
         apr_text_append(p, phdr, s);
     }
 }
@@ -93,22 +93,22 @@ DAV_DECLARE(int) dav_do_find_liveprop(const char *ns_uri, const char *name,
         if (strcmp(ns_uri, uris[ns]) == 0)
             break;
     if (uris[ns] == NULL) {
-	/* not our property (the namespace matched none of ours) */
-	return 0;
+        /* not our property (the namespace matched none of ours) */
+        return 0;
     }
 
     /* second: look for the property in the liveprop specs */
     for (scan = group->specs; scan->name != NULL; ++scan)
-	if (ns == scan->ns && strcmp(name, scan->name) == 0) {
+        if (ns == scan->ns && strcmp(name, scan->name) == 0) {
             *hooks = group->hooks;
-	    return scan->propid;
+            return scan->propid;
         }
 
     /* not our property (same namespace, but no matching prop name) */
     return 0;
 }
 
-DAV_DECLARE(int) dav_get_liveprop_info(int propid,
+DAV_DECLARE(long) dav_get_liveprop_info(int propid,
                                        const dav_liveprop_group *group,
                                        const dav_liveprop_spec **info)
 {
@@ -128,7 +128,7 @@ DAV_DECLARE(int) dav_get_liveprop_info(int propid,
     return 0;
 }
 
-DAV_DECLARE(void) dav_register_liveprop_group(apr_pool_t *p, 
+DAV_DECLARE(void) dav_register_liveprop_group(apr_pool_t *p,
                                               const dav_liveprop_group *group)
 {
     /* register the namespace URIs */
