@@ -1,9 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,9 +14,14 @@
  * limitations under the License.
  */
 
-/*
-** DAV extension module for Apache 2.0.*
-*/
+/**
+ * @file  mod_dav.h 
+ * @brief DAV extension module for Apache 2.0.*
+ *
+ * @defgroup MOD_DAV mod_dav
+ * @ingroup APACHE_MODS
+ * @{
+ */
 
 #ifndef _MOD_DAV_H_
 #define _MOD_DAV_H_
@@ -681,7 +686,8 @@ DAV_DECLARE(const dav_provider *) dav_lookup_provider(const char *name);
 typedef enum
 {
     dav_if_etag,
-    dav_if_opaquelock
+    dav_if_opaquelock,
+    dav_if_unknown /* the "unknown" state type; always matches false. */   
 } dav_if_state_type;
 
 typedef struct dav_if_state_list
@@ -876,19 +882,19 @@ DAV_DECLARE(int) dav_do_find_liveprop(const char *ns_uri, const char *name,
                                       const dav_hooks_liveprop **hooks);
 
 /* ### docco */
-DAV_DECLARE(int) dav_get_liveprop_info(int propid,
-                                       const dav_liveprop_group *group,
-                                       const dav_liveprop_spec **info);
+DAV_DECLARE(long) dav_get_liveprop_info(int propid,
+                                        const dav_liveprop_group *group,
+                                        const dav_liveprop_spec **info);
 
 /* ### docco */
 DAV_DECLARE(void) dav_register_liveprop_group(apr_pool_t *pool, 
                                               const dav_liveprop_group *group);
 
 /* ### docco */
-DAV_DECLARE(int) dav_get_liveprop_ns_index(const char *uri);
+DAV_DECLARE(long) dav_get_liveprop_ns_index(const char *uri);
 
 /* ### docco */
-DAV_DECLARE(int) dav_get_liveprop_ns_count(void);
+DAV_DECLARE(long) dav_get_liveprop_ns_count(void);
 
 /* ### docco */
 DAV_DECLARE(void) dav_add_all_liveprop_xmlns(apr_pool_t *p,
@@ -2418,3 +2424,5 @@ typedef struct {
 #endif
 
 #endif /* _MOD_DAV_H_ */
+/** @} */
+
