@@ -346,7 +346,7 @@ static int show_server_settings(request_rec * r)
     ap_rprintf(r,
                "<dl><dt><strong>Server Version:</strong> "
                "<font size=\"+1\"><tt>%s</tt></font></dt>\n",
-               ap_get_server_version());
+               ap_get_server_description());
     ap_rprintf(r,
                "<dt><strong>Server Built:</strong> "
                "<font size=\"+1\"><tt>%s</tt></font></dt>\n",
@@ -607,7 +607,7 @@ static int display_info(request_rec * r)
     if (r->method_number != M_GET)
         return DECLINED;
 
-    ap_set_content_type(r, "text/html");
+    ap_set_content_type(r, "text/html; charset=ISO-8859-1");
 
     ap_rputs(DOCTYPE_XHTML_1_0T
              "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
@@ -721,7 +721,7 @@ static int display_info(request_rec * r)
                                            ap_escape_html(r->pool, cmd->name),
                                            cmd->name[0] == '<' ? "&gt;" : "");
                                 if (cmd->errmsg) {
-                                    ap_rputs(cmd->errmsg, r);
+                                    ap_rputs(ap_escape_html(r->pool, cmd->errmsg), r);
                                 }
                                 ap_rputs("</i></tt></dd>\n", r);
                             }
