@@ -1,9 +1,9 @@
-/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -13,7 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
- 
+
 #define CORE_PRIVATE
 #define INCL_NOPMAPI
 #define INCL_DOS
@@ -25,7 +25,7 @@
 #include "http_main.h"
 #include "http_log.h"
 #include "http_config.h"
-#include "http_core.h"		/* for get_remote_host */
+#include "http_core.h"  /* for get_remote_host */
 #include "http_connection.h"
 #include "mpm.h"
 #include "ap_mpm.h"
@@ -39,8 +39,8 @@
 
 /* XXXXXX move these to header file private to this MPM */
 
-/* We don't need many processes, 
- * they're only for redundancy in the event of a crash 
+/* We don't need many processes,
+ * they're only for redundancy in the event of a crash
  */
 #define HARD_SERVER_LIMIT 10
 
@@ -81,7 +81,7 @@ static void server_maintenance(void *vpArg);
 static void clean_child_exit(int code)
 {
     if (pchild) {
-	apr_pool_destroy(pchild);
+        apr_pool_destroy(pchild);
     }
 
     exit(code);
@@ -92,15 +92,13 @@ static void clean_child_exit(int code)
 void ap_mpm_child_main(apr_pool_t *pconf)
 {
     ap_listen_rec *lr = NULL;
-    ap_listen_rec *first_lr = NULL;
     int requests_this_child = 0;
-    apr_socket_t *sd = ap_listeners->sd;
-    int nsds, rv = 0;
+    int rv = 0;
     unsigned long ulTimes;
     int my_pid = getpid();
     ULONG rc, c;
     HQUEUE workq;
-    apr_pollfd_t *pollset;
+    apr_pollset_t *pollset;
     int num_listeners;
     TID server_maint_tid;
     void *sb_mem;
@@ -181,24 +179,34 @@ void ap_mpm_child_main(apr_pool_t *pconf)
         num_listeners++;
     }
 
-    apr_poll_setup(&pollset, num_listeners, pchild);
+    apr_pollset_create(&pollset, num_listeners, pchild, 0);
 
-    for (lr = ap_listeners; lr; lr = lr->next) {
-        apr_poll_socket_add(pollset, lr->sd, APR_POLLIN);
+    for (lr = ap_listeners; lr != NULL; lr = lr->next) {
+        apr_pollfd_t pfd = { 0 };
+
+        pfd.desc_type = APR_POLL_SOCKET;
+        pfd.desc.s = lr->sd;
+        pfd.reqevents = APR_POLLIN;
+        pfd.client_data = lr;
+        apr_pollset_add(pollset, &pfd);
     }
 
     /* Main connection accept loop */
     do {
         apr_pool_t *pconn;
         worker_args_t *worker_args;
+        int last_poll_idx = 0;
 
         apr_pool_create(&pconn, pchild);
         worker_args = apr_palloc(pconn, sizeof(worker_args_t));
         worker_args->pconn = pconn;
 
         if (num_listeners == 1) {
-            rv = apr_accept(&worker_args->conn_sd, ap_listeners->sd, pconn);
+            rv = apr_socket_accept(&worker_args->conn_sd, ap_listeners->sd, pconn);
         } else {
+            const apr_pollfd_t *poll_results;
+            apr_int32_t num_poll_results;
+
             rc = DosRequestMutexSem(ap_mpm_accept_mutex, SEM_INDEFINITE_WAIT);
 
             if (shutdown_pending) {
@@ -209,50 +217,25 @@ void ap_mpm_child_main(apr_pool_t *pconf)
             rv = APR_FROM_OS_ERROR(rc);
 
             if (rv == APR_SUCCESS) {
-                rv = apr_poll(pollset, num_listeners, &nsds, -1);
+                rv = apr_pollset_poll(pollset, -1, &num_poll_results, &poll_results);
                 DosReleaseMutexSem(ap_mpm_accept_mutex);
             }
 
             if (rv == APR_SUCCESS) {
-                if (first_lr == NULL) {
-                    first_lr = ap_listeners;
+                if (last_poll_idx >= num_listeners) {
+                    last_poll_idx = 0;
                 }
 
-                lr = first_lr;
-
-                do {
-                    apr_int16_t event;
-
-                    apr_poll_revents_get(&event, lr->sd, pollset);
-
-                    if (event == APR_POLLIN) {
-                        apr_sockaddr_t *sa;
-                        apr_port_t port;
-                        apr_socket_addr_get(&sa, APR_LOCAL, lr->sd);
-                        apr_sockaddr_port_get(&port, sa);
-                        first_lr = lr->next;
-                        break;
-                    }
-                    lr = lr->next;
-
-                    if (!lr) {
-                        lr = ap_listeners;
-                    }
-                } while (lr != first_lr);
-
-                if (lr == first_lr) {
-                    continue;
-                }
-
-                sd = lr->sd;
-                rv = apr_accept(&worker_args->conn_sd, sd, pconn);
+                lr = poll_results[last_poll_idx++].client_data;
+                rv = apr_socket_accept(&worker_args->conn_sd, lr->sd, pconn);
+                last_poll_idx++;
             }
         }
 
         if (rv != APR_SUCCESS) {
             if (!APR_STATUS_IS_EINTR(rv)) {
                 ap_log_error(APLOG_MARK, APLOG_ERR, rv, ap_server_conf,
-                             "apr_accept");
+                             "apr_socket_accept");
                 clean_child_exit(APEXIT_CHILDFATAL);
             }
         } else {
@@ -349,7 +332,7 @@ ULONG APIENTRY thread_exception_handler(EXCEPTIONREPORTRECORD *pReportRec,
         kill(getpid(), SIGHUP);
         DosUnwindException(UNWIND_ALL, 0, 0);
     }
-  
+
     return XCPT_CONTINUE_SEARCH;
 }
 
@@ -372,7 +355,7 @@ static void worker_main(void *vpArg)
     int thread_slot = (int)vpArg;
     EXCEPTIONREGISTRATIONRECORD reg_rec = { NULL, thread_exception_handler };
     ap_sb_handle_t *sbh;
-  
+
     /* Trap exceptions in this thread so we don't take down the whole process */
     DosSetExceptionHandler( &reg_rec );
 
@@ -386,7 +369,7 @@ static void worker_main(void *vpArg)
     }
 
     conn_id = ID_FROM_CHILD_THREAD(child_slot, thread_slot);
-    ap_update_child_status_from_indexes(child_slot, thread_slot, SERVER_READY, 
+    ap_update_child_status_from_indexes(child_slot, thread_slot, SERVER_READY,
                                         NULL);
 
     apr_allocator_create(&allocator);
@@ -407,11 +390,11 @@ static void worker_main(void *vpArg)
         }
 
         apr_pool_destroy(pconn);
-        ap_update_child_status_from_indexes(child_slot, thread_slot, 
+        ap_update_child_status_from_indexes(child_slot, thread_slot,
                                             SERVER_READY, NULL);
     }
 
-    ap_update_child_status_from_indexes(child_slot, thread_slot, SERVER_DEAD, 
+    ap_update_child_status_from_indexes(child_slot, thread_slot, SERVER_DEAD,
                                         NULL);
 
     apr_bucket_alloc_destroy(bucket_alloc);
@@ -488,10 +471,10 @@ static void set_signals()
     sa.sa_handler = sig_term;
 
     if (sigaction(SIGTERM, &sa, NULL) < 0)
-	ap_log_error(APLOG_MARK, APLOG_WARNING, errno, ap_server_conf, "sigaction(SIGTERM)");
+        ap_log_error(APLOG_MARK, APLOG_WARNING, errno, ap_server_conf, "sigaction(SIGTERM)");
 
     sa.sa_handler = sig_hup;
 
     if (sigaction(SIGHUP, &sa, NULL) < 0)
-	ap_log_error(APLOG_MARK, APLOG_WARNING, errno, ap_server_conf, "sigaction(SIGHUP)");
+        ap_log_error(APLOG_MARK, APLOG_WARNING, errno, ap_server_conf, "sigaction(SIGHUP)");
 }
