@@ -1,9 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -67,7 +67,7 @@ static apr_filetype_e filetype_from_mode(mode_t mode)
     return type;
 }
 
-static void fill_out_finfo(apr_finfo_t *finfo, struct stat *info,
+static void fill_out_finfo(apr_finfo_t *finfo, struct_stat *info,
                            apr_int32_t wanted)
 { 
     finfo->valid = APR_FINFO_MIN | APR_FINFO_IDENT | APR_FINFO_NLINK
@@ -81,21 +81,69 @@ static void fill_out_finfo(apr_finfo_t *finfo, struct stat *info,
     finfo->device = info->st_dev;
     finfo->nlink = info->st_nlink;
     apr_time_ansi_put(&finfo->atime, info->st_atime);
+#ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
+    finfo->atime += info->st_atim.tv_nsec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_ATIMENSEC)
+    finfo->atime += info->st_atimensec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_ATIME_N)
+    finfo->ctime += info->st_atime_n / APR_TIME_C(1000);
+#endif
+
     apr_time_ansi_put(&finfo->mtime, info->st_mtime);
+#ifdef HAVE_STRUCT_STAT_ST_MTIM_TV_NSEC
+    finfo->mtime += info->st_mtim.tv_nsec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_MTIMENSEC)
+    finfo->mtime += info->st_mtimensec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_MTIME_N)
+    finfo->ctime += info->st_mtime_n / APR_TIME_C(1000);
+#endif
+
     apr_time_ansi_put(&finfo->ctime, info->st_ctime);
-    /* ### needs to be revisited  
-     * if (wanted & APR_FINFO_CSIZE) {
-     *   finfo->csize = info->st_blocks * 512;
-     *   finfo->valid |= APR_FINFO_CSIZE;
-     * }
-     */
+#ifdef HAVE_STRUCT_STAT_ST_CTIM_TV_NSEC
+    finfo->ctime += info->st_ctim.tv_nsec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_CTIMENSEC)
+    finfo->ctime += info->st_ctimensec / APR_TIME_C(1000);
+#elif defined(HAVE_STRUCT_STAT_ST_CTIME_N)
+    finfo->ctime += info->st_ctime_n / APR_TIME_C(1000);
+#endif
+
+#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
+#ifdef DEV_BSIZE
+    finfo->csize = (apr_off_t)info->st_blocks * (apr_off_t)DEV_BSIZE;
+#else
+    finfo->csize = (apr_off_t)info->st_blocks * (apr_off_t)512;
+#endif
+    finfo->valid |= APR_FINFO_CSIZE;
+#endif
+}
+
+apr_status_t apr_file_info_get_locked(apr_finfo_t *finfo, apr_int32_t wanted,
+                                      apr_file_t *thefile)
+{
+    struct_stat info;
+
+    if (thefile->buffered) {
+        apr_status_t rv = apr_file_flush_locked(thefile);
+        if (rv != APR_SUCCESS)
+            return rv;
+    }
+
+    if (fstat(thefile->filedes, &info) == 0) {
+        finfo->pool = thefile->pool;
+        finfo->fname = thefile->fname;
+        fill_out_finfo(finfo, &info, wanted);
+        return (wanted & ~finfo->valid) ? APR_INCOMPLETE : APR_SUCCESS;
+    }
+    else {
+        return errno;
+    }
 }
 
 APR_DECLARE(apr_status_t) apr_file_info_get(apr_finfo_t *finfo, 
                                             apr_int32_t wanted,
                                             apr_file_t *thefile)
 {
-    struct stat info;
+    struct_stat info;
 
     if (thefile->buffered) {
         apr_status_t rv = apr_file_flush(thefile);
@@ -138,7 +186,7 @@ APR_DECLARE(apr_status_t) apr_file_attrs_set(const char *fname,
         return APR_SUCCESS;
 
     status = apr_stat(&finfo, fname, APR_FINFO_PROT, pool);
-    if (!APR_STATUS_IS_SUCCESS(status))
+    if (status)
         return status;
 
     /* ### TODO: should added bits be umask'd? */
@@ -188,7 +236,7 @@ APR_DECLARE(apr_status_t) apr_file_mtime_set(const char *fname,
     apr_finfo_t finfo;
 
     status = apr_stat(&finfo, fname, APR_FINFO_ATIME, pool);
-    if (!APR_STATUS_IS_SUCCESS(status)) {
+    if (status) {
         return status;
     }
 
@@ -228,7 +276,7 @@ APR_DECLARE(apr_status_t) apr_stat(apr_finfo_t *finfo,
                                    const char *fname, 
                                    apr_int32_t wanted, apr_pool_t *pool)
 {
-    struct stat info;
+    struct_stat info;
     int srv;
 
     if (wanted & APR_FINFO_LINK)
@@ -279,11 +327,4 @@ APR_DECLARE(apr_status_t) apr_stat(apr_finfo_t *finfo,
     }
 }
 
-/* Perhaps this becomes nothing but a macro?
- */
-APR_DECLARE(apr_status_t) apr_lstat(apr_finfo_t *finfo, const char *fname,
-                      apr_int32_t wanted, apr_pool_t *pool)
-{
-    return apr_stat(finfo, fname, wanted | APR_FINFO_LINK, pool);
-}
 
