@@ -284,6 +284,11 @@ char url[1024];
 char * fullurl, * colonhost;
 int isproxy = 0;
 apr_interval_time_t aprtimeout = apr_time_from_sec(30); /* timeout value */
+
+/* overrides for ab-generated common headers */
+int opt_host = 0;       /* was an optional "Host:" header specified? */
+int opt_useragent = 0;  /* was an optional "User-Agent:" header specified? */
+int opt_accept = 0;     /* was an optional "Accept:" header specified? */
  /*
   * XXX - this is now a per read/write transact type of value
   */
@@ -731,7 +736,7 @@ static void output_results(void)
     printf("\n\n");
     printf("Server Software:        %s\n", servername);
     printf("Server Hostname:        %s\n", hostname);
-    printf("Server Port:            %hd\n", port);
+    printf("Server Port:            %hu\n", port);
 #ifdef USE_SSL
     if (is_ssl && ssl_info) {
         printf("SSL/TLS Protocol:       %s\n", ssl_info);
@@ -990,7 +995,7 @@ static void output_html_results(void)
        "<td colspan=2 %s>%s</td></tr>\n",
        trstring, tdstring, tdstring, hostname);
     printf("<tr %s><th colspan=2 %s>Server Port:</th>"
-       "<td colspan=2 %s>%hd</td></tr>\n",
+       "<td colspan=2 %s>%hu</td></tr>\n",
        trstring, tdstring, tdstring, port);
     printf("<tr %s><th colspan=2 %s>Document Path:</th>"
        "<td colspan=2 %s>%s</td></tr>\n",
@@ -1142,7 +1147,7 @@ static void start_connect(struct connection * c)
         SSL_set_connect_state(c->ssl);
         if (verbosity >= 4) {
             BIO_set_callback(bio, ssl_print_cb);
-            BIO_set_callback_arg(bio, bio_err);
+            BIO_set_callback_arg(bio, (void *)bio_err);
         }
     } else {
         c->ssl = NULL;
@@ -1512,37 +1517,54 @@ static void test(void)
         apr_err("apr_pollset_create failed", status);
     }
 
+    /* add default headers if necessary */
+    if (!opt_host) {
+        /* Host: header not overridden, add default value to hdrs */
+        hdrs = apr_pstrcat(cntxt, hdrs, "Host: ", host_field, colonhost, "\r\n", NULL);
+    }
+    else {
+        /* Header overridden, no need to add, as it is already in hdrs */
+    }
+
+    if (!opt_useragent) {
+        /* User-Agent: header not overridden, add default value to hdrs */
+        hdrs = apr_pstrcat(cntxt, hdrs, "User-Agent: ApacheBench/", AP_AB_BASEREVISION, "\r\n", NULL);
+    }
+    else {
+        /* Header overridden, no need to add, as it is already in hdrs */
+    }
+
+    if (!opt_accept) {
+        /* Accept: header not overridden, add default value to hdrs */
+        hdrs = apr_pstrcat(cntxt, hdrs, "Accept: */*\r\n", NULL);
+    }
+    else {
+        /* Header overridden, no need to add, as it is already in hdrs */
+    }
+
     /* setup request */
     if (posting <= 0) {
         snprintf_res = apr_snprintf(request, sizeof(_request),
             "%s %s HTTP/1.0\r\n"
-            "User-Agent: ApacheBench/%s\r\n"
             "%s" "%s" "%s"
-            "Host: %s%s\r\n"
-            "Accept: */*\r\n"
             "%s" "\r\n",
             (posting == 0) ? "GET" : "HEAD",
             (isproxy) ? fullurl : path,
-            AP_AB_BASEREVISION,
             keepalive ? "Connection: Keep-Alive\r\n" : "",
-            cookie, auth, host_field, colonhost, hdrs);
+            cookie, auth, hdrs);
     }
     else {
         snprintf_res = apr_snprintf(request,  sizeof(_request),
             "POST %s HTTP/1.0\r\n"
-            "User-Agent: ApacheBench/%s\r\n"
             "%s" "%s" "%s"
-            "Host: %s%s\r\n"
-            "Accept: */*\r\n"
             "Content-length: %" APR_SIZE_T_FMT "\r\n"
             "Content-type: %s\r\n"
             "%s"
             "\r\n",
             (isproxy) ? fullurl : path,
-            AP_AB_BASEREVISION,
             keepalive ? "Connection: Keep-Alive\r\n" : "",
             cookie, auth,
-            host_field, colonhost, postlen,
+            postlen,
             (content_type[0]) ? content_type : "text/plain", hdrs);
     }
     if (snprintf_res >= sizeof(_request)) {
@@ -2035,6 +2057,16 @@ int main(int argc, const char * const argv[])
                 break;
             case 'H':
                 hdrs = apr_pstrcat(cntxt, hdrs, optarg, "\r\n", NULL);
+                /*
+                 * allow override of some of the common headers that ab adds
+                 */
+                if (strncasecmp(optarg, "Host:", 5) == 0) {
+                    opt_host = 1;
+                } else if (strncasecmp(optarg, "Accept:", 7) == 0) {
+                    opt_accept = 1;
+                } else if (strncasecmp(optarg, "User-Agent:", 11) == 0) {
+                    opt_useragent = 1;
+                }
                 break;
             case 'w':
                 use_html = 1;
